From 5f3eeb0510423b6a51e63d06f957908113dcc4a1 Mon Sep 17 00:00:00 2001
From: Peter Leonov <gojpeg@gmail.com>
Date: Sun, 10 Jan 2010 07:12:07 +0300
Subject: [PATCH] directives listing

Adds "-l" directive. List all known directives (and their internal
types when doubled).

It might be useful building a table of Nginx directives with
information about their contexts, argument numbers and so on.
Generating a simple config file syntax highlighting rules
based on its output could be much simplier than manual.

---
 src/core/nginx.c |   44 ++++++++++++++++++++++++++++++++++++++++++++
 1 files changed, 44 insertions(+), 0 deletions(-)

diff --git a/src/core/nginx.c b/src/core/nginx.c
index 5df96a4..83ba120 100644
--- a/src/core/nginx.c
+++ b/src/core/nginx.c
@@ -186,6 +186,7 @@ ngx_uint_t          ngx_max_module;
 static ngx_uint_t   ngx_show_help;
 static ngx_uint_t   ngx_show_version;
 static ngx_uint_t   ngx_show_configure;
+static ngx_uint_t   ngx_show_directives = 0;
 static u_char      *ngx_prefix;
 static u_char      *ngx_conf_file;
 static u_char      *ngx_conf_params;
@@ -232,6 +233,7 @@ main(int argc, char *const *argv)
                                    NGX_CONF_PATH ")" CRLF
                 "  -g directives : set global directives out of configuration "
                                    "file" CRLF
+                "  -l,-ll        : list all known directives (and their types) " CRLF
                 );
         }
 
@@ -249,6 +251,39 @@ main(int argc, char *const *argv)
             ngx_log_stderr(0, "configure arguments:" NGX_CONFIGURE);
         }
 
+        if (ngx_show_directives) {
+            ngx_command_t      *cmd;
+            ngx_core_module_t  *module;
+            for (i = 0; ngx_modules[i]; i++) {
+                if (ngx_modules[i]->type == NGX_CORE_MODULE) {
+                    module = ngx_modules[i]->ctx;
+                    if (module == NULL) {
+                        ngx_log_stderr(0, "NULL");
+                    } else if (module->name.len > 0 && module->name.len < 75) {
+                        ngx_log_stderr(0, "%*s", module->name.len, module->name.data);
+                    } else {
+                        ngx_log_stderr(0, "unnamed");
+                    }
+                } else {
+                    ngx_log_stderr(0, "?");
+                }
+                
+                cmd = ngx_modules[i]->commands;
+                if (cmd == NULL) {
+                    continue;
+                }
+                
+                for ( /* void */ ; cmd->name.len; cmd++) {
+                    if (ngx_show_directives == 2)
+                    {
+                        ngx_log_stderr(0, "  %s %d", cmd->name.data, cmd->type);
+                    } else {
+                        ngx_log_stderr(0, "  %s", cmd->name.data);
+                    }
+                }
+            }
+        }
+
         if (!ngx_test_config) {
             return 0;
         }
@@ -751,6 +786,15 @@ ngx_get_options(int argc, char *const *argv)
                 ngx_log_stderr(0, "invalid option: \"-s %s\"", ngx_signal);
                 return NGX_ERROR;
 
+            case 'l':
+                ngx_show_version = 1;
+                if (++ngx_show_directives > 2) {
+                    ngx_log_stderr(0, "invalid number of \"-l\" options");
+                    return NGX_ERROR;
+                }
+                
+                break;
+
             default:
                 ngx_log_stderr(0, "invalid option: \"%c\"", *(p - 1));
                 return NGX_ERROR;
-- 
1.6.4.3

